/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/

#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QMap>
#include <QVector>

#include <QGLWidget>
#include <QGLFunctions>
#include <QMouseEvent>
#include <QTimer>

#include "RveRawdata.h"
#include "RveModel.h"
#include "RveIsoModel.h"
#include "RveFlatModel.h"
#include "RveMap.h"

/*---------------------------------------------------------------------------*/
/*!
\class
\brief
*/
/*---------------------------------------------------------------------------*/
class GLWidget : public QGLWidget, QGLFunctions
{
   Q_OBJECT

public:
   GLWidget( QWidget *parent = 0 );
   ~GLWidget();

   RveFlatModel *flatModel();
   void enableFlatModel();
   bool flatModelEnabled();
   void setFlatParameters( RveFlatParameters parameters );
   RveFlatParameters flatParameters();

   RveCartesianModel *cartesianModel();
   void enableCartesianModel();
   bool cartesianModelEnabled();
   void setCartesianParameters( RveCartesianParameters parameters );
   RveCartesianParameters cartesianParameters();

   RveIsoModel *isoModel();
   void enableIsoModel();
   bool isoModelEnabled();
   void setIsoParameters( RveIsoParameters parameters );
   RveIsoParameters isoParameters();

   RveMap *map();

   void enter2DMode();
   void leave2DMode();

   void project3DTo2D(
      double xPos, double yPos, double zPos,
      double &resX, double &resY, double &resZ );

public slots:
   void onNewRawdata( RveRawdata *pRawdata );
   void render();

protected:
   void initializeGL();
   void paintGL();
   void resizeGL( int width, int height );
   void mousePressEvent( QMouseEvent *event );
   void mouseReleaseEvent( QMouseEvent *event );
   void mouseMoveEvent( QMouseEvent *event );
   void wheelEvent( QWheelEvent *event );

private:
   void clearModels();
   void setData( RveRawdata *pRawdata );

   qint64 m_MSecs;
   qint64 m_MouseMoveMSecs;

   // *** Flat Model
   RveFlatModel *m_pFlatModel;
   // *** Cartesian Model
   RveCartesianModel *m_pCartesianModel;
   // *** Isometric Model
   RveIsoModel *m_pIsoModel;

   RveMap *m_pMap;
   RveRawdata *m_pRawdata;

   QTimer *m_pTimer;

   bool m_LeftButtonPressed;
   int m_MouseX;
   int m_MouseY;
   double m_Scale;
   double m_xRot;
   double m_yRot;
   double m_zRot;

   double m_Dx;
   double m_Dy;
};

#endif
