/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/

#ifndef RVECARTESIANMODEL_H
#define RVECARTESIANMODEL_H

#include <QObject>
#include <QMap>
#include <QGLFunctions>

#include "RveModel.h"
#include "RveRawdata.h"

#include "RveCartesianParameters.h"

/*---------------------------------------------------------------------------*/
/*!
\class
\brief
*/
/*---------------------------------------------------------------------------*/
class RveCartesianModel : public RveModel
{
   Q_OBJECT

public:
   RveCartesianModel( bool keepData = false );
   ~RveCartesianModel();

   RveCartesianParameters parameters() const;
   void setParameters( RveCartesianParameters parameters );

   virtual void setRawdata( RveRawdata *pRawdata );

   virtual void enable();
   virtual void disable();
   virtual bool isEnabled() const;

   virtual void paintGL();

   QVector<Vertex> &vertices();
   QVector<double> &data();
   double layerWidth() const;

private:
   void buildModel();
   void buildModel( bool keepData );
   void clearModel();

   void buildGLModel();
   void clearGLModel();

   void buildVertexModel();
   void buildVertexModel( bool keepData );
   void clearVertexModel();

   void buildVertexModel( double dh, bool keepData = false );

   double getValue( double dh, double dx, double dy );

   bool m_Enabled;

   bool m_KeepData;
   double m_Width;

   RveCartesianParameters m_Parameters;

   // The raw flat model
   QVector<double> m_Data;
   QVector<Vertex> m_Vertices;
   QVector<int>    m_Indices;

   // For OpenGL
   GLuint m_GLIndices;
   GLuint m_GLVertices;
};

#endif
