/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/
#include <QVector>

#include "RveGLTexture.h"
#include "GLWidget.h"


RveGLTexture::RveGLTexture( GLWidget *pGLWidget, const QImage &image ) :
   QGLFunctions( pGLWidget->context() ),
   m_pGLWidget( pGLWidget ),
   m_Width( image.width() ),
   m_Height( image.height() ),
   m_Texture( -1 )
{
   initializeGLFunctions();
   glGenTextures( 1, &m_Texture );
   glBindTexture( GL_TEXTURE_2D, m_Texture );
   QVector<quint8> pixelData;
   for( int y = 0; y < image.height(); y++ )
   {
      for( int x = 0; x < image.width(); x++ )
      {
         QRgb rgb = image.pixel( x, y );
         pixelData.append( qRed( rgb ) );
         pixelData.append( qGreen( rgb ) );
         pixelData.append( qBlue( rgb ) );
         pixelData.append( qAlpha( rgb ) );
      }
   }
   glTexImage2D( GL_TEXTURE_2D,
                 0,
                 GL_RGBA8,
                 image.width(), image.height(),
                 0,
                 GL_RGBA,
                 GL_UNSIGNED_BYTE,
                 pixelData.constData() );
   glGenerateMipmap( GL_TEXTURE_2D );
   glBindTexture( GL_TEXTURE_2D, 0 );
}


RveGLTexture::~RveGLTexture()
{
   glDeleteTextures( 1, &m_Texture );
}


GLuint RveGLTexture::glTextureID() const
{
   return( m_Texture );
}


int RveGLTexture::width() const
{
   return( m_Width );
}


int RveGLTexture::height() const
{
   return( m_Height );
}
