/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/

#ifndef RVEGISDATA_H
#define RVEGISDATA_H

#include <QObject>
#include <QVector>
#include <QDate>
#include <QFont>
#include <QGLFunctions>

#include "RveDbfFile.h"
#include "RveShpFile.h"
#include "RveProjPnt.h"
#include "RveModel.h"

/*---------------------------------------------------------------------------*/
/*!
\class
\brief
*/
/*---------------------------------------------------------------------------*/
class RveGisData : public QObject, public QGLFunctions
{
   Q_OBJECT

   public:
      RveGisData( const QString &fileName, QObject *parent = 0 );
      ~RveGisData();

      bool isValid() const;
      QString fileName() const;
      QColor color() const;
      void setColor( const QColor &color );
      int lineWidth() const;
      void setLineWidth( int lineWidth );
      bool isEnabled() const;
      void setEnabled( bool e );
      QFont font() const;
      void setFont( const QFont &font );
      void setFontScale( double scale );
      double fontScale() const;
      QString label() const;
      void setLabel( const QString &label );

      void calculate( const RveProjPnt &projPnt, double range_m, QVector<RveModel::Vertex> &map, int precision );

      QString filterFieldName() const;
      void setFilterFieldName( const QString &name );
      bool valueFiltered( const QVariant &value ) const;
      bool valueFiltered( int n ) const;
      void setValueFiltered( const QVariant &value, bool add );
      void clearFilterValues();

      const RveShpFile *shpFile() const;
      const RveDbfFile *dbfFile() const;

   signals:
      void changed();

   protected:
      bool load( const QString &fileName );
      void clearGL();

   private:
      bool m_Valid;
      bool m_Enabled;
      QString m_FileName;
      RveShpFile *m_pShpFile;
      RveDbfFile *m_pDbfFile;
      QFont m_Font;
      double m_FontScale;
      QString m_Label;
      QString m_FilterFieldName;
      QList<QVariant> m_FilterValues;

      QVector<GLuint> m_GLVertices;
      QVector<int>   m_GLNumVertices;
};

#endif
