/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/

#ifndef RVESHP_H
#define RVESHP_H

#include <QObject>
#include <QGLFunctions>
#include <QVector>
#include <QByteArray>
#include <QPainter>

#include "RveProjPnt.h"

/*---------------------------------------------------------------------------*/
/*!
\class
\brief
*/
/*---------------------------------------------------------------------------*/
class RveShp : public QObject
{
   Q_OBJECT

   public:
      enum SHP_TYPE
      {
         TYPE_NULL_SHAPE = 0,
         TYPE_POINT = 1,
         TYPE_POLYLINE = 3,
         TYPE_POLYGON = 5,
         TYPE_MULTIPOINT = 8,
         TYPE_POINTZ = 11,
         TYPE_POLYLINEZ = 13,
         TYPE_POLYGONZ = 15,
         TYPE_MULTIPOINTZ = 18,
         TYPE_POINTM = 21,
         TYPE_POLYLINEM = 23,
         TYPE_POLYGONM = 25,
         TYPE_MULTIPOINTM = 28,
         TYPE_MULTIPATCH = 31
      };

      typedef struct
      {
         double lon;
         double lat;
      } Point;

      ~RveShp();

      virtual bool valid() const;
      virtual quint32 recordNumber() const;

      virtual void calculate( QPainter &painter, const RveProjPnt &projPnt, double range_m, int precision ) = 0;

      virtual SHP_TYPE type() const;

   protected:
      RveShp( quint32 recordNumber, QObject *pParent = 0 );
 
      bool m_Valid;
      SHP_TYPE m_ShapeType;
      quint32 m_RecordNumber;

   private:
};

#endif
