/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/

#ifndef RVESHPFILE_H
#define RVESHPFILE_H

#include <QObject>
#include <QGLFunctions>
#include <QVector>

#include "RveShp.h"
#include "RveProjPnt.h"
#include "RveModel.h"

/*---------------------------------------------------------------------------*/
/*!
\class
\brief
*/
/*---------------------------------------------------------------------------*/
class RveShpFile : public QObject
{
   Q_OBJECT

   public:
      enum SHP_TYPE
      {
         TYPE_NULL_SHAPE = 0,
         TYPE_POINT = 1,
         TYPE_POLYLINE = 3,
         TYPE_POLYGON = 5,
         TYPE_MULTIPOINT = 8,
         TYPE_POINTZ = 11,
         TYPE_POLYLINEZ = 13,
         TYPE_POLYGONZ = 15,
         TYPE_MULTIPOINTZ = 18,
         TYPE_POINTM = 21,
         TYPE_POLYLINEM = 23,
         TYPE_POLYGONM = 25,
         TYPE_MULTIPOINTM = 28,
         TYPE_MULTIPATCH = 31
      };

      RveShpFile( const QString &fileName, QObject *parent = 0 );
      ~RveShpFile();

      void calculate( const RveProjPnt &projPnt, double range_m, QVector<RveModel::Vertex> &map, int precision );
      QString fileName() const;
      QColor color() const;
      void setColor( const QColor &color );
      int lineWidth() const;
      void setLineWidth( int lineWidth );
      bool isEnabled() const;
      void setEnabled( bool e );

      SHP_TYPE type() const;
      int numShapes() const;
      const RveShp *shape( int n ) const;

      bool isValid() const;

      static double toDouble( const QByteArray &p, int offset, bool bigEndian = false );
      static quint32 toUint32( const QByteArray &p, int offset, bool bigEndian = false );

   signals:
      void changed();

   protected:
      bool load( const QString &fileName );

   private:
      bool m_Valid;
      QString m_FileName;
      QColor m_Color;
      int m_LineWidth;
      bool m_Enabled;
      QVector<RveShp *> m_Shapes;
      quint32 m_FileCode;
      quint32 m_Version;
      SHP_TYPE m_ShapeType;
      double m_xMin;
      double m_yMin;
      double m_xMax;
      double m_yMax;
};

#endif
