/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/

#ifndef RveHDF5Compound_H
#define RveHDF5Compound_H

#include <QString>
#include <QVariant>

#include "RveHDF5Object.h"
#include "RveHDF5Attribute.h"

#include <hdf5.h>


/*----------------------------------------------------------------------------*/
/*!
\file   RveHDF5Compound.h
\author Christian Nowak
\class  RveHDF5Compound
\brief  Class representing an HDF5 compound dataset
*/
/*----------------------------------------------------------------------*/


class RveHDF5Compound : public RveHDF5Object
{
   public:
      typedef struct
      {
         QString name;
         RveHDF5Attribute::ATTR_TYPE attrType;
         size_t offset;
         size_t size;
      } CompoundType;

      RveHDF5Compound( QString name, hid_t dataset, RveHDF5Object *parent = 0 );
      RveHDF5Compound( QString name, void *data, size_t nItems, size_t itemSize );
      ~RveHDF5Compound();
      size_t size();
      void *data();
      QVariant data( const QString &name, int n ) const;
      RveHDF5Object::OBJ_TYPE type();
      RveHDF5Attribute::ATTR_TYPE dataType();
      size_t itemSize() const;
      size_t numItems() const;

      void addCompoundType( QString name, RveHDF5Attribute::ATTR_TYPE attrType, size_t offset, size_t size = 0 );
      QStringList compoundItems() const;

   protected:
      virtual bool save( hid_t f );

   private:
      bool load( hid_t dset );

      void *m_pData;
      size_t m_NumItems;
      size_t m_ItemSize;

      QVector<CompoundType> m_CompoundTypes;
};

#endif
