/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/
#ifndef RveHDF5Dataset_H
#define RveHDF5Dataset_H

#include <QString>

#include "RveHDF5Object.h"
#include "RveHDF5Attribute.h"

#include <hdf5.h>


class RveHDF5Dataset : public RveHDF5Object
{
   public:
      RveHDF5Dataset( QString name, hid_t dataset, RveHDF5Object *parent = 0 );
      RveHDF5Dataset( QString name, RveHDF5Attribute::ATTR_TYPE type, int nDims, int *dimensionality, void *data, int typeSize );
      ~RveHDF5Dataset();
      size_t size();
      void *data();
      int dim( int n );
      int numDimensions();
      RveHDF5Object::OBJ_TYPE type();
      RveHDF5Attribute::ATTR_TYPE dataType();
      int dataTypeSize();
      void print();

      bool add( RveHDF5Attribute *obj );
      RveHDF5Attribute *get( QString name );
      QStringList names();
      QVector<RveHDF5Attribute *> attributes();
      RveHDF5Attribute *getByPath( QString path );
      RveHDF5Attribute *getByPath( const QStringList &paths );

   protected:
      virtual bool save( hid_t f );

   private:
      bool load( hid_t dset );

      int m_NumDims;
      int *m_pDimensionality;
      size_t m_NumBytes;
      void *m_pData;
      RveHDF5Attribute::ATTR_TYPE m_Type;
      int m_TypeSize;
};

#endif

