/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/
#ifndef RveHDF5Object_H
#define RveHDF5Object_H

#include <QMap>
#include <QString>
#include <QVector>
#include <QVariant>

#include <hdf5.h>

class RveHDF5Object
{
   public:
      enum OBJ_TYPE
      {
         TYPE_ATTRIBUTE,
         TYPE_GROUP,
         TYPE_DATASET,
         TYPE_LINK,
         TYPE_COMPOUND
      };

      RveHDF5Object();
      RveHDF5Object( QString name, RveHDF5Object *parent = 0 );
      virtual ~RveHDF5Object();

      QString name();
      QString path();
      virtual OBJ_TYPE type() = 0;
      virtual void print();
      static bool load( QString fname, RveHDF5Object **obj );
      virtual bool save( QString fname );
      virtual bool add( RveHDF5Object *obj );
      virtual bool remove( QString name );
      RveHDF5Object *getByPath( QString path ) const;
      RveHDF5Object *getByPath( const QStringList &paths ) const;

      virtual bool getAttrString( QString path, QString &str ) const;
      virtual bool getAttrFloat( QString path, double &flt ) const;
      virtual bool getAttrInteger( QString path, int &i ) const;
      virtual bool getAttrString( const QStringList &paths, QString &str ) const;
      virtual bool getAttrFloat( const QStringList &paths, double &flt ) const;
      virtual bool getAttrInteger( const QStringList &paths, int &i ) const;

   protected:
      virtual bool save( hid_t f );
      void setParent( RveHDF5Object *parent );
      RveHDF5Object *get( QString name ) const;
      QStringList names();
      QVector<RveHDF5Object *> objects();

      QMap<QString, RveHDF5Object *> m_Objects;
      QString m_Name;
      RveHDF5Object *m_pParent;

   private:
};

#endif
