/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/
#ifndef RVEISOMODEL_H
#define RVEISOMODEL_H

#include <QObject>
#include <QMap>
#include <QGLFunctions>

#include "RveModel.h"
#include "RveCartesianModel.h"
#include "RveRawdata.h"
#include "RveIsoParameters.h"

/*---------------------------------------------------------------------------*/
/*!
\class
\brief
*/
/*---------------------------------------------------------------------------*/
class RveIsoModel : public RveModel
{
   Q_OBJECT

public:
   RveIsoModel();
   ~RveIsoModel();

   void setParameters( RveIsoParameters parameters );
   RveIsoParameters parameters() const;

   virtual void setRawdata( RveRawdata *pRawdata );

   virtual void enable();
   virtual void disable();
   virtual bool isEnabled() const;

   QVector<double> isoLevels();

   virtual void paintGL();
   void paintGL( double isolevel );

private:
   typedef struct {
      XYZ p[3];
   } TRIANGLE;

   typedef struct {
      XYZ p[8];
      double val[8];
   } GRIDCELL;

   void buildModel();
   void buildModel( double isolevel );
   void clearModel();
   void clearModel( double isolevel );

//   void buildGLModel();
   void buildGLModel( double isolevel );
   void clearGLModel();
   void clearGLModel( double isolevel );

//   void buildVertexModel();
   void buildVertexModel( double isolevel );
   void clearVertexModel();
   void clearVertexModel( double isolevel );

   static int Polygonise(GRIDCELL grid,double isolevel,TRIANGLE *triangles);
   static XYZ RveIsoModel::VertexInterp(
      double isolevel, XYZ p1, XYZ p2, double valp1, double valp2);



   void buildVertexModel( double isolevel, RveModel::RGBA color );
   void polygonise( double isolevel, RveModel::RGBA color );
   void polygonise(
      double isolevel,
      RveModel::RGBA color,
      int x, int y, int z );

   RveCartesianModel *m_pCartesianModel;
   RveIsoParameters m_Parameters;
   bool m_Enabled;

   QMap<double, QVector<Vertex> > m_Models;
   QMap<double, QVector<int> > m_ModelsIndices;
   QMap<double, GLuint> m_GLModels;
   QMap<double, GLuint> m_GLModelsIndices;
};

#endif
