/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/
#ifndef RVEISOPARAMETERS_H
#define RVEISOPARAMETERS_H

#include <QObject>
#include <QMap>
#include <QVector>
#include <QGLFunctions>

#include "RveModel.h"
#include "RveCartesianParameters.h"

/*---------------------------------------------------------------------------*/
/*!
\class
\brief
*/
/*---------------------------------------------------------------------------*/
class RveIsoParameters : public RveCartesianParameters
{
public:
   enum IsoStyle
   {
      Wireframe,
      Flat,
      Lighted
   };

   typedef struct
   {
      IsoStyle isoStyle;
      int      opacity;
      struct
      {
         double r;
         double g;
         double b;
      } color;
      double   isoLevel;
   } IsoParameters;

   RveIsoParameters(
      int nCellsPerLayer,
      int nLayers,
      double lowestLayer,
      double highestLayer );
   RveIsoParameters();
   ~RveIsoParameters();

   virtual bool valid() const;
   virtual bool equals( const RveIsoParameters &other ) const;
   virtual bool operator==( const RveIsoParameters &other ) const;
   virtual bool operator!=( const RveIsoParameters &other ) const;
   virtual RveIsoParameters &operator=( const RveIsoParameters &d );

   QVector<double> isoLevels() const;
   const IsoParameters isoParameters( double isoLevel ) const;
   bool hasIsoLevel( double l ) const;

   void addIsoLevel( double isoLevel, IsoStyle style, int opacity, double r, double g, double b );
   void addIsoLevel( RveIsoParameters::IsoParameters parameters );
   void removeIsoLevel( double isoLevel );

protected:
   void copy( const RveIsoParameters &other );
   //   IsoLevel
   QMap<double, IsoParameters> m_IsoParameters;
};

#endif
