/*----------------------------------------------------------------------------*/
/*!
Copyright (c) 2018, 2019 by Christian Nowak a.k.a. Creestee a.k.a. chn
Contact: chnowak@web.de or creestee@creestee.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
3. This application may be used and redistributed in source code and binary
form free of charge for personal and for educational purposes.
4. Redistribution and usage of this application in source code or binary form
for commercial purposes requires explicit written permission from the
author.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the RadarVolumeExplorer project.
*/
/*----------------------------------------------------------------------------*/

#include <QDoubleSpinBox>
#include <QCheckBox>
#include <QHeaderView>
#include <QPushButton>
#include <QFileDialog>
#include <QCoreApplication>

#include "RveLayersGuiHandler.h"
#include "RveLayersGuiOverlayItemHandler.h"

#include "RveGisData/RveGisData.h"
#include "RveShapeFileSettings.h"


/*---------------------------------------------------------------------------*/
/*!*/
/*---------------------------------------------------------------------------*/
RveLayersGuiHandler::RveLayersGuiHandler( QTreeWidget *twLayers, RveMap *pMap, QObject *parent ) :
   QObject( parent ),
   m_pTwLayers( twLayers ),
   m_pMap( pMap ),
   m_pRawdata( 0 )
{
   m_pTwLayers->header()->setResizeMode( QHeaderView::ResizeToContents );

   // **** Total Range
   QTreeWidgetItem *twiTotalRange = new QTreeWidgetItem();
   m_pTwLayers->addTopLevelItem( twiTotalRange );
   twiTotalRange->setText( 0, "Total Range" );
   m_pSbTotalRange = new QDoubleSpinBox();
   m_pSbTotalRange->setSuffix( "x" );
   m_pSbTotalRange->setValue( 1.0 );
   m_pSbTotalRange->setMinimum( 0.5 );
   m_pSbTotalRange->setMaximum( 8.0 );
   m_pSbTotalRange->setButtonSymbols( QAbstractSpinBox::NoButtons );
   m_pTwLayers->setItemWidget( twiTotalRange, 1, m_pSbTotalRange );

   // **** Direction Indicators
   QTreeWidgetItem *twiDirectionIndicators = new QTreeWidgetItem();
   twiDirectionIndicators->setText( 0, "Direction Indicators" );
   m_pTwLayers->addTopLevelItem( twiDirectionIndicators );
   // Enable
   QTreeWidgetItem *twiDirectionIndicatorsEnable = new QTreeWidgetItem();
   twiDirectionIndicatorsEnable->setText( 0, "Enable" );
   twiDirectionIndicators->addChild( twiDirectionIndicatorsEnable );
   m_pCbDirectionIndicatorsEnable = new QCheckBox();
   m_pTwLayers->setItemWidget( twiDirectionIndicatorsEnable, 1, m_pCbDirectionIndicatorsEnable );

   // **** Range Rings
   QTreeWidgetItem *twiRangeRings = new QTreeWidgetItem();
   twiRangeRings->setText( 0, "Range Rings" );
   m_pTwLayers->addTopLevelItem( twiRangeRings );
   // Enable
   QTreeWidgetItem *twiRangeRingsEnable = new QTreeWidgetItem();
   twiRangeRingsEnable->setText( 0, "Enable" );
   twiRangeRings->addChild( twiRangeRingsEnable );
   m_pCbRangeRingsEnable = new QCheckBox();
   m_pTwLayers->setItemWidget( twiRangeRingsEnable, 1, m_pCbRangeRingsEnable );
   // Distance
   QTreeWidgetItem *twiRangeRingsDistance = new QTreeWidgetItem();
   twiRangeRingsDistance->setText( 0, "Distance" );
   twiRangeRings->addChild( twiRangeRingsDistance );
   m_pSbRangeRingsDistance = new QDoubleSpinBox();
   m_pSbRangeRingsDistance->setSuffix( "km" );
   m_pSbRangeRingsDistance->setValue( 25.0 );
   m_pSbRangeRingsDistance->setMinimum( 0.5 );
   m_pSbRangeRingsDistance->setMaximum( 100.0 );
   m_pSbRangeRingsDistance->setButtonSymbols( QAbstractSpinBox::NoButtons );
   m_pTwLayers->setItemWidget( twiRangeRingsDistance, 1, m_pSbRangeRingsDistance );

   // **** Orographical Map
   QTreeWidgetItem *twiOroMap = new QTreeWidgetItem();
   twiOroMap->setText( 0, "Orographical Map" );
   m_pTwLayers->addTopLevelItem( twiOroMap );
   // Enable
   QTreeWidgetItem *twiOroMapEnable = new QTreeWidgetItem();
   twiOroMapEnable->setText( 0, "Enable" );
   twiOroMap->addChild( twiOroMapEnable );
   m_pCbOroMapEnable = new QCheckBox();
   m_pTwLayers->setItemWidget( twiOroMapEnable, 1, m_pCbOroMapEnable );

   // **** Height Grid
   QTreeWidgetItem *twiHeightGrid = new QTreeWidgetItem();
   twiHeightGrid->setText( 0, "Height Grid" );
   m_pTwLayers->addTopLevelItem( twiHeightGrid );
   // Enable
   QTreeWidgetItem *twiHeightGridEnable = new QTreeWidgetItem();
   twiHeightGridEnable->setText( 0, "Enable" );
   twiHeightGrid->addChild( twiHeightGridEnable );
   m_pCbHeightGridEnable = new QCheckBox();
   m_pTwLayers->setItemWidget( twiHeightGridEnable, 1, m_pCbHeightGridEnable );
   // Max Height
   QTreeWidgetItem *twiHeightGridMaxHeight = new QTreeWidgetItem();
   twiHeightGridMaxHeight->setText( 0, "Maximum Height" );
   twiHeightGrid->addChild( twiHeightGridMaxHeight );
   m_pSbHeightGridMaxHeight = new QDoubleSpinBox();
   m_pSbHeightGridMaxHeight->setSuffix( "km" );
   m_pSbHeightGridMaxHeight->setValue( 10.0 );
   m_pSbHeightGridMaxHeight->setMinimum( 1.0 );
   m_pSbHeightGridMaxHeight->setMaximum( 100.0 );
   m_pSbHeightGridMaxHeight->setSingleStep( 1.0 );
   m_pSbHeightGridMaxHeight->setButtonSymbols( QAbstractSpinBox::NoButtons );
   m_pTwLayers->setItemWidget( twiHeightGridMaxHeight, 1, m_pSbHeightGridMaxHeight );
   // Distance
   QTreeWidgetItem *twiHeightGridDistance = new QTreeWidgetItem();
   twiHeightGridDistance->setText( 0, "Distance" );
   twiHeightGrid->addChild( twiHeightGridDistance );
   m_pSbHeightGridDistance = new QDoubleSpinBox();
   m_pSbHeightGridDistance->setSuffix( "km" );
   m_pSbHeightGridDistance->setValue( 1.0 );
   m_pSbHeightGridDistance->setMinimum( 0.1 );
   m_pSbHeightGridDistance->setMaximum( 20.0 );
   m_pSbHeightGridDistance->setSingleStep( 0.5 );
   m_pSbHeightGridDistance->setButtonSymbols( QAbstractSpinBox::NoButtons );
   m_pTwLayers->setItemWidget( twiHeightGridDistance, 1, m_pSbHeightGridDistance );

   // **** Overlay
   m_pTwiOverlay = new QTreeWidgetItem();
   m_pTwiOverlay->setText( 0, "Overlay" );
   QPushButton *pbOverlayAdd = new QPushButton();
   pbOverlayAdd->setText( "+" );
   pbOverlayAdd->setStyleSheet( "border: none;" );
   pbOverlayAdd->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum ) );
   m_pTwLayers->addTopLevelItem( m_pTwiOverlay );
   m_pTwLayers->setItemWidget( m_pTwiOverlay, 1, pbOverlayAdd );
   connect( pbOverlayAdd, SIGNAL( clicked() ), this, SLOT( onPbOverlayAdd() ) );

   m_pTwLayers->expandAll();

   // Total Range
   connect( m_pSbTotalRange,                SIGNAL( editingFinished() ),
            this,                           SLOT(   onSbTotalRange() ) );
   // Oromap
   connect( m_pCbOroMapEnable,              SIGNAL( toggled( bool ) ),
            this,                           SLOT(   onCbOroMapEnable( bool ) ) );
   // Range Rigns
   connect( m_pCbRangeRingsEnable,          SIGNAL( toggled( bool ) ),
            this,                           SLOT(   onCbRangeRingsEnable( bool ) ) );
   connect( m_pSbRangeRingsDistance,        SIGNAL( editingFinished() ),
            this,                           SLOT(   onSbRangeRingsDistance() ) );
   // Directions
   connect( m_pCbDirectionIndicatorsEnable, SIGNAL( toggled( bool ) ),
            this,                           SLOT(   onCbDirectionIndicatorsEnable( bool ) ) );
   // Height Grid
   connect( m_pCbHeightGridEnable,          SIGNAL( toggled( bool ) ),
            this,                           SLOT(   onCbHeightGridEnable( bool ) ) );
   connect( m_pSbHeightGridMaxHeight,       SIGNAL( editingFinished() ),
            this,                           SLOT(   onSbHeightGridMaxHeight() ) );
   connect( m_pSbHeightGridDistance,        SIGNAL( editingFinished() ),
            this,                           SLOT(   onSbHeightGridDistance() ) );

   updateGui();
}


/*---------------------------------------------------------------------------*/
/*!*/
/*---------------------------------------------------------------------------*/
RveLayersGuiHandler::~RveLayersGuiHandler()
{
}


void RveLayersGuiHandler::onPbOverlayAdd()
{
   QString fileName = QFileDialog::getOpenFileName(
      0,
      "Please Select a Shape File",
      QCoreApplication::applicationDirPath() + QDir::separator() + "..",
      QString( "Shape Files (*.shp)" ) );

   if( !fileName.isEmpty() )
   {
      RveGisData *pGisData = new RveGisData( fileName, m_pMap );

      if( !pGisData->isValid() )
      {
         delete pGisData;
         return;
      }

      if( pGisData->shpFile()->type() == RveShpFile::TYPE_POINT )
      {
         RveShapeFileSettings *pSettings = new RveShapeFileSettings( pGisData, m_pTwLayers );
         pSettings->show();
         pSettings->exec();
         delete pSettings;
      }

      // Add an entry
      RveLayersGuiOverlayItemHandler *pOverlayItemHandler =
         new RveLayersGuiOverlayItemHandler( this, m_pTwLayers, m_pTwiOverlay, pGisData, this );
      connect( pOverlayItemHandler, SIGNAL( remove( RveLayersGuiOverlayItemHandler * ) ),
               this, SLOT( onRemoveOverlayItem( RveLayersGuiOverlayItemHandler * ) ) );
      m_pMap->addGisData( pGisData );
      m_OverlayItems.append( pOverlayItemHandler );
      updateGui();

      connect( pGisData, SIGNAL( changed() ), m_pMap, SLOT( calculate() ) );
   }
}


int RveLayersGuiHandler::numEnabledItems() const
{
   int result = 0;
   foreach( RveLayersGuiOverlayItemHandler *pItem, m_OverlayItems )
   {
      if( pItem->isEnabled() )
         result++;
   }

   return( result );
}


void RveLayersGuiHandler::onRemoveOverlayItem( RveLayersGuiOverlayItemHandler *item )
{
   m_OverlayItems.removeAll( item );
   m_pMap->removeGisData( item->gisData() );
   delete item;
}


void RveLayersGuiHandler::updateGui()
{
   if( m_pRawdata )
   {
      m_pTwLayers->setEnabled( true );

      // Layers
      m_pCbOroMapEnable->setEnabled( true );
      m_pCbDirectionIndicatorsEnable->setEnabled( true );
      m_pCbRangeRingsEnable->setEnabled( true );
      m_pCbHeightGridEnable->setEnabled( true );

      m_pSbRangeRingsDistance->setEnabled( m_pCbRangeRingsEnable->isChecked() );
      m_pSbHeightGridMaxHeight->setEnabled( m_pCbHeightGridEnable->isChecked() );
      m_pSbHeightGridDistance->setEnabled( m_pCbHeightGridEnable->isChecked() );

      m_pSbTotalRange->setEnabled(
         m_pCbOroMapEnable->isChecked() ||
         m_pCbDirectionIndicatorsEnable->isChecked() ||
         m_pCbRangeRingsEnable->isChecked() ||
         numEnabledItems() > 0 );
   } else
   {
      m_pTwLayers->setEnabled( false );

      // Layers
      m_pCbOroMapEnable->setEnabled( false );
      m_pCbDirectionIndicatorsEnable->setEnabled( false );
      m_pCbRangeRingsEnable->setEnabled( false );
      m_pCbHeightGridEnable->setEnabled( false );

      m_pSbRangeRingsDistance->setEnabled( false );
      m_pSbHeightGridMaxHeight->setEnabled( false );
      m_pSbHeightGridDistance->setEnabled( false );

      m_pSbTotalRange->setEnabled( false );
   }
}


void RveLayersGuiHandler::onSbTotalRange()
{
   m_pMap->setRange( m_pRawdata->range() * m_pSbTotalRange->value() );
   emit updateGL();
}


void RveLayersGuiHandler::onCbOroMapEnable( bool enabled )
{
   m_pMap->enableOromap( enabled );
   updateGui();
   emit updateGL();
}


void RveLayersGuiHandler::onCbRangeRingsEnable( bool enabled )
{
   m_pMap->enableRangeRings( enabled );
   updateGui();
   emit updateGL();
}


void RveLayersGuiHandler::onSbRangeRingsDistance()
{
   m_pMap->setRangeRingsDistance( m_pSbRangeRingsDistance->value() * 1000.0 );
   emit updateGL();
}


void RveLayersGuiHandler::onCbDirectionIndicatorsEnable( bool enabled )
{
   m_pMap->enableDirections( enabled );
   updateGui();
   emit updateGL();
}


void RveLayersGuiHandler::onCbHeightGridEnable( bool enabled )
{
   m_pMap->enableHeightGrid( enabled );
   updateGui();
   emit updateGL();
}


void RveLayersGuiHandler::onSbHeightGridMaxHeight()
{
   m_pMap->setHeightGridMaxHeight( m_pSbHeightGridMaxHeight->value() * 1000.0 );
   emit updateGL();
}


void RveLayersGuiHandler::onSbHeightGridDistance()
{
   m_pMap->setHeightGridDistance( m_pSbHeightGridDistance->value() * 1000.0 );
   emit updateGL();
}


void RveLayersGuiHandler::onNewRawdata( RveRawdata *pRawdata )
{
   setRawdata( pRawdata );
}

void RveLayersGuiHandler::setRawdata( RveRawdata *pRawdata )
{
   if( m_pRawdata == pRawdata )
      return;

   m_pRawdata = pRawdata;

   m_pMap->setRange( m_pSbTotalRange->value() * m_pRawdata->range() );
   m_pMap->setRangeRingsDistance( m_pSbRangeRingsDistance->value() * 1000.0 );
   m_pMap->enableOromap( m_pCbOroMapEnable->isChecked() );
   m_pMap->enableRangeRings( m_pCbRangeRingsEnable->isChecked() );
   m_pMap->enableHeightGrid( m_pCbHeightGridEnable->isChecked() );
   m_pMap->setHeightGridDistance( m_pSbHeightGridDistance->value() * 1000.0 );
   m_pMap->setHeightGridMaxHeight( m_pSbHeightGridMaxHeight->value() * 1000.0 );
   m_pMap->enableDirections( m_pCbDirectionIndicatorsEnable->isChecked() );

   updateGui();
}
